import asyncio
import pickle

from langchain_community.chat_models import ChatOllama

from ceylon.llm.types import Job, Step, AgentDefinition
from ceylon.llm.unit import LLMAgent, ChiefAgent
from ceylon.tools.search_tool import SearchTool


async def main():
    llm_lib = ChatOllama(model="llama3:instruct")

    chief = ChiefAgent()

    writer = LLMAgent(
        AgentDefinition(
            name="writer",
            role="AI Content Writer",
            role_description="Transform complex AI research into engaging, accessible content for a wide audience.",
            responsibilities=[
                "Write 800+ word articles on AI topics",
                "Simplify technical concepts with metaphors",
                "Create narrative-driven content",
                "Ensure scientific accuracy and readability",
            ],
            skills=[
                "Creative writing",
                "Technical simplification",
                "Storytelling",
                "Audience engagement",
            ],
            tools=["Metaphor generator", "Readability analyzer"],
            knowledge_domains=["AI", "Machine Learning", "Data Science"],
            interaction_style="Friendly and conversational, making complex ideas exciting.",
            operational_parameters="""
            Balance accuracy with creativity. Link analogies to AI concepts. Encourage critical thinking.
            Avoid prefatory statements about your capabilities or role. Begin directly with the requested content.
        """,
            performance_objectives=[
                "Increase AI topic engagement",
                "Improve public understanding of AI",
                "Bridge expert-public communication gap",
            ],
            version="3.1.0"
        ),
        llm=llm_lib
    )
    researcher = LLMAgent(
        AgentDefinition(
            name="researcher",
            role="AI and Machine Learning Research Specialist",
            role_description="Your role is to gather detailed and accurate information on how AI can be utilized in machine learning...",
            responsibilities=[
                "Conduct thorough research on AI applications in machine learning",
                "Gather detailed information from reputable academic and industry sources",
            ],
            skills=[
                "Advanced information retrieval and data mining",
                "Critical analysis of technical papers and reports",
            ],
            tools=[
                "Academic database access (e.g., arXiv, IEEE Xplore)",
                "Industry report aggregators",
            ],
            knowledge_domains=[
                "Artificial Intelligence",
                "Machine Learning Algorithms",
            ],
            interaction_style="Professional and analytical. Communicate findings clearly and objectively, with a focus on accuracy and relevance.",
            operational_parameters="Prioritize peer-reviewed sources and reputable industry reports...",
            performance_objectives=[
                "Provide comprehensive coverage of AI applications in machine learning",
                "Ensure all gathered information is current and accurately represented",
            ],
            version="2.0.0"
        ),
        tools=[SearchTool()],
        llm=llm_lib
    )

    proof_writer = LLMAgent(
        AgentDefinition(
            name="proof_writer",
            role="Content Editor and Finalizer",
            role_description="Refine and optimize AI-generated articles, ensuring they are publication-ready.",
            responsibilities=[
                "Edit for clarity, coherence, and flow",
                "Correct grammar, punctuation, and spelling",
                "Enhance the article structure and formatting",
                "Ensure consistent tone and style",
                "Add or revise titles, subtitles, and section headings",
                "Implement SEO best practices",
                "Create a compelling conclusion",
                "Generate meta descriptions and keywords",
            ],
            skills=[
                "Advanced editing",
                "Proofreading",
                "SEO optimization",
                "Content structuring",
                "Style consistency maintenance",
            ],
            tools=["Grammar checker", "SEO analysis tool", "Readability scorer"],
            knowledge_domains=["Content Editing", "SEO", "Web Publishing", "Writing Styles"],
            interaction_style="Professional and detail-oriented, with a focus on enhancing readability and engagement.",
            operational_parameters="""
                  Maintain the original voice and intent of the content.
                  Optimize for both human readers and search engines.
                  Ensure all edits align with the target audience's expectations.
                  Do not add new information or change the core message of the article.
                  Focus on polishing and refining rather than rewriting.
                """,
            performance_objectives=[
                "Improve overall article quality and readability",
                "Enhance SEO performance of the content",
                "Ensure the article meets publication standards",
                "Maintain the engaging and accessible style of the original content",
            ],
            version="1.0.0"
        ), llm=llm_lib)

    job = Job(
        title="write_article",
        work_order=[
            Step(owner="researcher", dependencies=[]),
            Step(owner="writer", dependencies=["researcher"]),
            Step(owner="proof_writer", dependencies=["writer"]),
        ],
        input={
            "title": "How to use AI for Machine Learning",
            "tone": "informal",
            "style": "creative"
        }
    )

    res = await chief.run_admin(pickle.dumps(job), [
        writer,
        researcher,
        proof_writer
    ])
    print("Response:", res)


if __name__ == '__main__':
    # enable_log("INFO")
    asyncio.run(main())
